from setuptools import setup

setup(name='cellworld_experiment_service',description='experiment service',author='german espinosa',author_email='germanespinosa@gmail.com',packages=['cellworld_experiment_service'],install_requires=['cellworld', 'tcp-messages'],license='MIT',version='0.0.2',zip_safe=False)
